@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2010  Intel Corporation.
  * Copyright (C) 2011  Robert Bosch Car Multimedia GmbH.
- * Copyright (C) 2012, 2014  Collabora Ltd.
+ * Copyright (C) 2012  Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -61,7 +61,6 @@
 
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
-#include "clutter-gesture-action-private.h"
 #include "clutter-marshal.h"
 #include "clutter-private.h"
 #include <math.h>
@@ -80,14 +79,6 @@ typedef enum
   PAN_STATE_INTERPOLATING
 } PanState;
 
-typedef enum
-{
-  SCROLL_PINNED_UNKNOWN,
-  SCROLL_PINNED_NONE,
-  SCROLL_PINNED_HORIZONTAL,
-  SCROLL_PINNED_VERTICAL
-} PinState;
-
 struct _ClutterPanActionPrivate
 {
   ClutterPanAxis pan_axis;
@@ -110,8 +101,6 @@ struct _ClutterPanActionPrivate
   gfloat release_y;
 
   guint should_interpolate : 1;
-
-  PinState pin_state;
 };
 
 enum
@@ -138,41 +127,15 @@ enum
 
 static guint pan_signals[LAST_SIGNAL] = { 0, };
 
-G_DEFINE_TYPE_WITH_PRIVATE (ClutterPanAction, clutter_pan_action, CLUTTER_TYPE_GESTURE_ACTION)
+G_DEFINE_TYPE (ClutterPanAction, clutter_pan_action,
+               CLUTTER_TYPE_GESTURE_ACTION);
 
 static void
 emit_pan (ClutterPanAction *self,
           ClutterActor     *actor,
           gboolean          is_interpolated)
 {
-  ClutterPanActionPrivate *priv = self->priv;
   gboolean retval;
-
-  if (priv->pin_state == SCROLL_PINNED_UNKNOWN)
-    {
-      priv->pin_state = SCROLL_PINNED_NONE;
-      if (priv->pan_axis == CLUTTER_PAN_AXIS_AUTO)
-        {
-          gfloat delta_x;
-          gfloat delta_y;
-          gfloat scroll_threshold = G_PI_4/2;
-          gfloat drag_angle;
-
-          clutter_gesture_action_get_motion_delta (CLUTTER_GESTURE_ACTION (self), 0, &delta_x, &delta_y);
-
-          if (delta_x != 0.0f)
-            drag_angle = atanf (delta_y / delta_x);
-          else
-            drag_angle = G_PI_2;
-
-          if ((drag_angle > -scroll_threshold) && (drag_angle < scroll_threshold))
-            priv->pin_state = SCROLL_PINNED_HORIZONTAL;
-          else if ((drag_angle > (G_PI_2 - scroll_threshold)) ||
-                    (drag_angle < -(G_PI_2 - scroll_threshold)))
-            priv->pin_state = SCROLL_PINNED_VERTICAL;
-        }
-    }
-
   g_signal_emit (self, pan_signals[PAN], 0, actor, is_interpolated, &retval);
 }
 
@@ -225,8 +188,8 @@ on_deceleration_new_frame (ClutterTimeline     *timeline,
 }
 
 static gboolean
-gesture_prepare (ClutterGestureAction  *gesture,
-                 ClutterActor          *actor)
+gesture_begin (ClutterGestureAction  *gesture,
+               ClutterActor          *actor)
 {
   ClutterPanAction *self = CLUTTER_PAN_ACTION (gesture);
   ClutterPanActionPrivate *priv = self->priv;
@@ -234,17 +197,6 @@ gesture_prepare (ClutterGestureAction  *gesture,
   if (priv->state == PAN_STATE_INTERPOLATING && priv->deceleration_timeline)
     clutter_timeline_stop (priv->deceleration_timeline);
 
-  return TRUE;
-}
-
-static gboolean
-gesture_begin (ClutterGestureAction  *gesture,
-               ClutterActor          *actor)
-{
-  ClutterPanAction *self = CLUTTER_PAN_ACTION (gesture);
-  ClutterPanActionPrivate *priv = self->priv;
-
-  priv->pin_state = SCROLL_PINNED_UNKNOWN;
   priv->state = PAN_STATE_PANNING;
   priv->interpolated_x = priv->interpolated_y = 0.0f;
   priv->dx = priv->dy = 0.0f;
@@ -335,10 +287,28 @@ clutter_pan_action_real_pan (ClutterPanAction *self,
                              ClutterActor     *actor,
                              gboolean          is_interpolated)
 {
+  ClutterPanActionPrivate *priv = self->priv;
   gfloat dx, dy;
   ClutterMatrix transform;
 
-  clutter_pan_action_get_constrained_motion_delta (self, 0, &dx, &dy);
+  if (is_interpolated)
+    clutter_pan_action_get_interpolated_delta (self, &dx, &dy);
+  else
+    clutter_gesture_action_get_motion_delta (CLUTTER_GESTURE_ACTION (self), 0, &dx, &dy);
+
+  switch (priv->pan_axis)
+    {
+    case CLUTTER_PAN_AXIS_NONE:
+      break;
+
+    case CLUTTER_PAN_X_AXIS:
+      dy = 0.0f;
+      break;
+
+    case CLUTTER_PAN_Y_AXIS:
+      dx = 0.0f;
+      break;
+    }
 
   clutter_actor_get_child_transform (actor, &transform);
   cogl_matrix_translate (&transform, dx, dy, 0.0f);
@@ -409,15 +379,6 @@ clutter_pan_action_get_property (GObject    *gobject,
     }
 }
 
-static void
-clutter_pan_action_constructed (GObject *gobject)
-{
-  ClutterGestureAction *gesture;
-
-  gesture = CLUTTER_GESTURE_ACTION (gobject);
-  clutter_gesture_action_set_threshold_trigger_edge (gesture, CLUTTER_GESTURE_TRIGGER_EDGE_AFTER);
-}
-
 static void
 clutter_pan_action_dispose (GObject *gobject)
 {
@@ -456,9 +417,10 @@ clutter_pan_action_class_init (ClutterPanActionClass *klass)
   ClutterGestureActionClass *gesture_class =
       CLUTTER_GESTURE_ACTION_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (ClutterPanActionPrivate));
+
   klass->pan = clutter_pan_action_real_pan;
 
-  gesture_class->gesture_prepare = gesture_prepare;
   gesture_class->gesture_begin = gesture_begin;
   gesture_class->gesture_progress = gesture_progress;
   gesture_class->gesture_cancel = gesture_cancel;
@@ -530,7 +492,6 @@ clutter_pan_action_class_init (ClutterPanActionClass *klass)
                          1.0, G_MAXDOUBLE, default_acceleration_factor,
                          CLUTTER_PARAM_READWRITE);
 
-  gobject_class->constructed = clutter_pan_action_constructed;
   gobject_class->set_property = clutter_pan_action_set_property;
   gobject_class->get_property = clutter_pan_action_get_property;
   gobject_class->dispose = clutter_pan_action_dispose;
@@ -539,7 +500,7 @@ clutter_pan_action_class_init (ClutterPanActionClass *klass)
                                       pan_props);
 
   /**
-   * ClutterPanAction::pan:
+   * ClutterPanAction:pan:
    * @action: the #ClutterPanAction that emitted the signal
    * @actor: the #ClutterActor attached to the @action
    * @is_interpolated: if the event is the result of interpolating
@@ -550,8 +511,8 @@ clutter_pan_action_class_init (ClutterPanActionClass *klass)
    * interpolation phase of the pan, after the drag has ended and
    * the :interpolate property was set to %TRUE.
    *
-   * Return value: %TRUE if the pan should continue, and %FALSE if
-   *   the pan should be cancelled.
+   * Return value: TRUE if the pan action has been handled by one of
+   * the listener or %FALSE to continue the emission.
    *
    * Since: 1.12
    */
@@ -567,7 +528,7 @@ clutter_pan_action_class_init (ClutterPanActionClass *klass)
                   G_TYPE_BOOLEAN);
 
   /**
-   * ClutterPanAction::pan-stopped:
+   * ClutterPanAction:pan-stopped:
    * @action: the #ClutterPanAction that emitted the signal
    * @actor: the #ClutterActor attached to the @action
    *
@@ -590,10 +551,12 @@ clutter_pan_action_class_init (ClutterPanActionClass *klass)
 static void
 clutter_pan_action_init (ClutterPanAction *self)
 {
-  self->priv = clutter_pan_action_get_instance_private (self);
-  self->priv->deceleration_rate = default_deceleration_rate;
-  self->priv->acceleration_factor = default_acceleration_factor;
-  self->priv->state = PAN_STATE_INACTIVE;
+  ClutterPanActionPrivate *priv = self->priv =
+    G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_PAN_ACTION,
+                                 ClutterPanActionPrivate);
+  priv->deceleration_rate = default_deceleration_rate;
+  priv->acceleration_factor = default_acceleration_factor;
+  priv->state = PAN_STATE_INACTIVE;
 }
 
 /**
@@ -628,7 +591,7 @@ clutter_pan_action_set_pan_axis (ClutterPanAction *self,
 
   g_return_if_fail (CLUTTER_IS_PAN_ACTION (self));
   g_return_if_fail (axis >= CLUTTER_PAN_AXIS_NONE &&
-                    axis <= CLUTTER_PAN_AXIS_AUTO);
+                    axis <= CLUTTER_PAN_Y_AXIS);
 
   priv = self->priv;
 
@@ -830,18 +793,16 @@ clutter_pan_action_get_interpolated_coords (ClutterPanAction *self,
  * Retrieves the delta, in stage space, since the latest interpolated
  * event, analogous to clutter_gesture_action_get_motion_delta().
  *
- * Return value: the distance since the latest interpolated event
- *
  * Since: 1.12
  */
-gfloat
+void
 clutter_pan_action_get_interpolated_delta (ClutterPanAction *self,
                                            gfloat           *delta_x,
                                            gfloat           *delta_y)
 {
   ClutterPanActionPrivate *priv;
 
-  g_return_val_if_fail (CLUTTER_IS_PAN_ACTION (self), 0.0f);
+  g_return_if_fail (CLUTTER_IS_PAN_ACTION (self));
 
   priv = self->priv;
 
@@ -850,170 +811,4 @@ clutter_pan_action_get_interpolated_delta (ClutterPanAction *self,
 
   if (delta_y)
     *delta_y = priv->dy;
-
-  return sqrt ((priv->dx * priv->dx) + (priv->dy * priv->dy));
-}
-
-/**
- * clutter_pan_action_get_constrained_motion_delta:
- * @self: A #ClutterPanAction
- * @point: the touch point index, with 0 being the first touch
-  *   point received by the action
- * @delta_x: (out) (optional): return location for the X delta
- * @delta_y: (out) (optional): return location for the Y delta
- *
- * Retrieves the delta, in stage space, dependent on the current state
- * of the #ClutterPanAction, and respecting the constraint specified by the
- * #ClutterPanAction:pan-axis property.
- *
- * Return value: the distance since last motion event
- *
- * Since: 1.24
- */
-gfloat
-clutter_pan_action_get_constrained_motion_delta (ClutterPanAction *self,
-                                                 guint             point,
-                                                 gfloat           *out_delta_x,
-                                                 gfloat           *out_delta_y)
-{
-  ClutterPanActionPrivate *priv;
-  gfloat delta_x = 0.f, delta_y = 0.f, distance;
-
-  g_return_val_if_fail (CLUTTER_IS_PAN_ACTION (self), 0.0f);
-
-  priv = self->priv;
-
-  distance = clutter_pan_action_get_motion_delta (self, point, &delta_x, &delta_y);
-
-  switch (priv->pan_axis)
-    {
-    case CLUTTER_PAN_AXIS_NONE:
-      break;
-
-    case CLUTTER_PAN_AXIS_AUTO:
-      if (priv->pin_state == SCROLL_PINNED_VERTICAL)
-        delta_x = 0.0f;
-      else if (priv->pin_state == SCROLL_PINNED_HORIZONTAL)
-        delta_y = 0.0f;
-      break;
-
-    case CLUTTER_PAN_X_AXIS:
-      delta_y = 0.0f;
-      break;
-
-    case CLUTTER_PAN_Y_AXIS:
-      delta_x = 0.0f;
-      break;
-    }
-
-  if (out_delta_x)
-    *out_delta_x = delta_x;
-
-  if (out_delta_y)
-    *out_delta_y = delta_y;
-
-  return distance;
-}
-
-/**
- * clutter_pan_action_get_motion_delta:
- * @self: A #ClutterPanAction
- * @point: the touch point index, with 0 being the first touch
- *   point received by the action
- * @delta_x: (out) (allow-none): return location for the X delta
- * @delta_y: (out) (allow-none): return location for the Y delta
- *
- * Retrieves the delta, in stage space, dependent on the current state
- * of the #ClutterPanAction. If it is inactive, both fields will be
- * set to 0. If it is panning by user action, the values will be equivalent
- * to those returned by clutter_gesture_action_get_motion_delta().
- * If it is interpolating with some form of kinetic scrolling, the values
- * will be equivalent to those returned by
- * clutter_pan_action_get_interpolated_delta(). This is a convenience
- * method designed to be used in replacement "pan" signal handlers.
- *
- * Since: 1.14
- */
-gfloat
-clutter_pan_action_get_motion_delta (ClutterPanAction *self,
-                                     guint             point,
-                                     gfloat           *delta_x,
-                                     gfloat           *delta_y)
-{
-  ClutterPanActionPrivate *priv;
-
-  g_return_val_if_fail (CLUTTER_IS_PAN_ACTION (self), 0.0f);
-
-  priv = self->priv;
-
-  switch (priv->state)
-    {
-    case PAN_STATE_INACTIVE:
-      if (delta_x)
-        *delta_x = 0;
-
-      if (delta_y)
-        *delta_y = 0;
-
-      return 0;
-    case PAN_STATE_PANNING:
-      return clutter_gesture_action_get_motion_delta (CLUTTER_GESTURE_ACTION (self),
-                                                      point, delta_x, delta_y);
-    case PAN_STATE_INTERPOLATING:
-      return clutter_pan_action_get_interpolated_delta (self, delta_x, delta_y);
-    default:
-      g_assert_not_reached ();
-    }
-}
-
-/**
- * clutter_pan_action_get_motion_coords:
- * @self: A #ClutterPanAction
- * @point: the touch point index, with 0 being the first touch
- *   point received by the action
- * @motion_x: (out) (allow-none): return location for the X coordinate
- * @motion_y: (out) (allow-none): return location for the Y coordinate
- *
- * Retrieves the coordinates, in stage space, dependent on the current state
- * of the #ClutterPanAction. If it is inactive, both fields will be
- * set to 0. If it is panning by user action, the values will be equivalent
- * to those returned by clutter_gesture_action_get_motion_coords().
- * If it is interpolating with some form of kinetic scrolling, the values
- * will be equivalent to those returned by
- * clutter_pan_action_get_interpolated_coords(). This is a convenience
- * method designed to be used in replacement "pan" signal handlers.
- *
- * Since: 1.14
- */
-void
-clutter_pan_action_get_motion_coords (ClutterPanAction *self,
-                                      guint             point,
-                                      gfloat           *motion_x,
-                                      gfloat           *motion_y)
-{
-  ClutterPanActionPrivate *priv;
-
-  g_return_if_fail (CLUTTER_IS_PAN_ACTION (self));
-
-  priv = self->priv;
-
-  switch (priv->state)
-    {
-    case PAN_STATE_INACTIVE:
-      if (motion_x)
-        *motion_x = 0;
-
-      if (motion_y)
-        *motion_y = 0;
-      break;
-    case PAN_STATE_PANNING:
-      clutter_gesture_action_get_motion_coords (CLUTTER_GESTURE_ACTION (self),
-                                                point, motion_x, motion_y);
-      break;
-    case PAN_STATE_INTERPOLATING:
-      clutter_pan_action_get_interpolated_coords (self, motion_x, motion_y);
-      break;
-    default:
-      g_assert_not_reached ();
-    }
 }
