@@ -891,7 +891,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                           "invalid",
                           event->scroll.x,
                           event->scroll.y,
-                          (xev->flags & XIPointerEmulated) ? "yes" : "no");
+#ifdef HAVE_XINPUT_2_2
+                          (xev->flags & XIPointerEmulated) ? "yes" : "no"
+#else
+                          "no"
+#endif
+                          );
             break;
 
           default:
@@ -939,7 +944,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                           event->button.x,
                           event->button.y,
                           event->button.axes != NULL ? "yes" : "no",
-                          (xev->flags & XIPointerEmulated) ? "yes" : "no");
+#ifdef HAVE_XINPUT_2_2
+                          (xev->flags & XIPointerEmulated) ? "yes" : "no"
+#else
+                          "no"
+#endif
+                          );
             break;
           }
 
@@ -1076,9 +1086,6 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                             stage_x11,
                                             &xev->valuators);
 
-        if (source_device != NULL && device->stage != NULL)
-          _clutter_input_device_set_stage (source_device, device->stage);
-
         if (xi_event->evtype == XI_TouchBegin)
           {
             event->touch.modifier_state |= CLUTTER_BUTTON1_MASK;
@@ -1129,9 +1136,6 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                             stage_x11,
                                             &xev->valuators);
 
-        if (source_device != NULL && device->stage != NULL)
-          _clutter_input_device_set_stage (source_device, device->stage);
-
         event->touch.modifier_state =
           _clutter_input_device_xi2_translate_state (&xev->mods,
                                                      &xev->buttons);
@@ -1175,7 +1179,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->crossing.x = xev->event_x;
             event->crossing.y = xev->event_y;
 
-            _clutter_stage_add_device (stage, device);
+            _clutter_input_device_set_stage (device, stage);
           }
         else
           {
@@ -1199,7 +1203,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
             event->crossing.x = xev->event_x;
             event->crossing.y = xev->event_y;
 
-            _clutter_stage_remove_device (stage, device);
+            _clutter_input_device_set_stage (device, NULL);
           }
 
         _clutter_input_device_reset_scroll_info (source_device);
